function [classes,W,objfunc,sparsity,RQvar,L1norm,dist]=RTR_2class(train,train_l,test,test_l,delta,lam,eps,start,classic)
% function [classes,W,objfunc,sparsity]=RTR_2class(train,train_l,test,test_l, delta, lambda, epsilon, start, classic)
% classifies two sets of points after a training phase. The classification
% hyperplanes are determinend minimiizing the sum of a generalized
% Rayleigh quotient with a twice continuously differentiable
% approximation (eps-L1) of a L1-regularization term.
%
% Input variables
% train:    (input)  contains all training points;
%
% train_l:  (input)  vector of classification labels of the training set.
%
% test:     (input)  contains all testing points;
%
% test_l:   (input)  vector of classification labels of the testing set.
%
% delta:    (input)  regularization parameter for the hyperplanes
%                    (usually 1e-03 | 1e-04 | 1e-05 )
%
% lambda:   (input)  weight of the regularization.
%
%
% epsilon:  (input)  parameter of the eps-L1 function.
%
% start:    (input)  if not null contains the starting points of the trust
%                    region algorithm.
%
% classic:  (input)  determines whether the classical or the "new"
%                    assignment criterion has to be used.
% 
% Output variables
% classes:  (output) vector of all evaluated classification labels.
%
% W:        (output) contains the coefficients of the two planes
%
% objfunc:  (output) contains the value of the objective functions at the solution
%                    of the minimum problems.
%
% sparsity: (output) contains the sparsity of the solution.
%
    global lambda;
    global epsilon;


    lambda  = lam;
    epsilon = eps;

    global CLASS;

    [train,mu,sigma] = zscore(train,1);
    for i=1:size(test,2)
        test(:,i)=(test(:,i)-mu(i))/sigma(i);
    end

    % load training data in A and B
    A=train(train_l==CLASS(1),:);
    B=train(train_l==CLASS(2),:);

    n=size(A,2);

    % Build the numerator and denominator matrices of the generalized
    % Rayleigh quotient using a gaussian kernel if required

    g=[A, -ones(size(A,1),1)];
    h=[B, -ones(size(B,1),1)];

    % Build planes
    if nargin<9
        if ((size(A,1)<size(A,2))||(size(B,1)<size(B,2)))
            classic=1;
        else
            classic=0;
        end
    end

    if max(size(start.obj))~=1
        params1.x0=start.W(:,1);
        [X1,fx1,RQvar1,L1norm1,dist1]=rtr_l1approx(g,h,delta,params1);
        params2.x0=start.W(:,2);
        [X2,fx2,RQvar2,L1norm2,dist2]=rtr_l1approx(h,g,delta,params2);
    else
        if max(size(start.W))~=1
            params1.max_outer=0;
            params1.x0=start.W(:,1);
            [X1,fx1,RQvar1,L1norm1,dist1]=rtr_l1approx(g,h,delta,params1);

            params2.max_outer=0;
            params2.x0=start.W(:,2);
            [X2,fx2,RQvar2,L1norm2,dist2]=rtr_l1approx(h,g,delta,params2);
        else
            [X1,fx1,RQvar1,L1norm1,dist1]=rtr_l1approx(g,h,delta);
            [X2,fx2,RQvar2,L1norm2,dist2]=rtr_l1approx(h,g,delta);
        end
    end
    
    W=[X1 X2];
    RQvar  = [RQvar1,RQvar2];

    L1norm = [fx1,fx2];
   dist = [dist1,dist2];

    global tol;
    reltol=1/sqrt(n+1)*tol;
    W(1:end-1,:) = W(1:end-1,:).*(abs(W(1:end-1,:))>=reltol);


    sparsity = sum(sum(abs(W(1:end-1,:))==0,2)==size(W,2))/(size(W,1)-1);


    objfunc = [fx1 fx2];


    % Classify test points

    if classic == 0     % %     WITH PROJECTION
        tempTRclass=zeros(size(train_l));
        for i=1:2
            tempTRclass(train_l==CLASS(i))=i;
        end
        tempTESTclass=PredictMRG(train,tempTRclass, W, test);
        classes=zeros(size(test_l));
        for i=1:2
            classes(tempTESTclass==i) = CLASS(i);
        end
    else                % %     CLASSICAL
        z1=(abs(test*W(1:n,1)-W(n+1,1)).^2)/(norm(W(1:n,1))^2);
        z2=(abs(test*W(1:n,2)-W(n+1,2)).^2)/(norm(W(1:n,2))^2);
        classes=zeros(size(test_l));
        for i=1:size(test,1)
            if(z1(i)<z2(i))
                classes(i)=CLASS(1);
            else
                classes(i)=CLASS(2);
            end
        end
    end
end